//===-- XCoreMetadata.h ----------------------------*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// Collect XCore specific metadata for a module.
//
//===----------------------------------------------------------------------===//

#ifndef XCOREMETADATA_H
#define XCOREMETADATA_H

#include "llvm/Pass.h"
#include <map>
#include <string>

namespace llvm {

//===----------------------------------------------------------------------===//
// Forward declarations.
class Module;
class GlobalValue;

class XCoreMetadata: public ImmutablePass {
private:
  std::map<const GlobalValue *, std::string> TypeStrings;
  std::map<const Function *, uint64_t> StackAttributes;

  void AnalyzeTypeStrings(Module &M);
  void AnalyzeStackAttributes(Module &M);
public:
  static char ID; // Pass identification, replacement for typeid

  XCoreMetadata();
  ~XCoreMetadata();

  const std::string *GetTypeString(const GlobalValue &GV);
  bool GetStackAttribute(const Function &F, uint64_t &Value);

  /// AnalyzeModule - Scan the module for typestring information.
  void AnalyzeModule(Module &M);
};
}

#endif // XCOREMETADATA_H
