//===-- XCoreResourceUseWriter.cpp ------------------------------*- C++ -*-===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is distributed under the University of Illinois Open Source
// License. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//
//
// Emit resource usage information.
//
//===----------------------------------------------------------------------===//

#ifndef XCORE_RESOURCE_USE_WRITER
#define XCORE_RESOURCE_USE_WRITER

#include <string>
#include <set>

namespace llvm {

// forward declarations
class AsmPrinter;
class TargetAsmInfo;
class MachineFunction;
class Function;
class XCoreMetadata;
class Module;
class raw_ostream;

class XCoreResourceUseWriter
{
private:
  /// O - Stream to .s file.
  ///
  raw_ostream &O;

  /// Asm - Target of resource use emission.
  ///
  AsmPrinter *Asm;
  
  /// TAI - Target Asm Printer.
  const TargetAsmInfo *TAI;

  /// Name of the current function
  std::string CurrentFnName;

  /// XCore specific metadata
  XCoreMetadata *Meta;

  enum {
    StackWords,
    MaxThreads,
    MaxTimers,
    MaxChanends
  };
  
  std::set<std::string> WeakSymbols;
  
  void EmitResourceSymbol(const char *name, unsigned Type);
  void EmitResourceSymbol(const std::string &name, unsigned Type);
  void EmitResourceSymbol(const Function *F, unsigned Type);
  std::string GetResourseSymbolName(const Function *F, unsigned Type);
  
  bool EmitUsage(MachineFunction &MF, unsigned Type);

public:
  XCoreResourceUseWriter(raw_ostream &OS, AsmPrinter *A,
                         const TargetAsmInfo *T);

  void SetMetadata(XCoreMetadata *M) {
    Meta = M;
  }

  void EmitFunction(MachineFunction &MF);
  
  void EndModule();
};

} // namespace llvm

#endif // XCORE_RESOURCE_USE_WRITER
